import argparse
import sys
import re
import os

from pathlib import Path
import shutil

def main():
    # parse arguments
    parser = argparse.ArgumentParser(description="parse netlist and pack all references to current working directory")
    parser.add_argument("files", nargs="*", type=argparse.FileType("r"))
    parser.add_argument("-d", action="store_const", const=True, default=False, help="Dryrun")

    args = parser.parse_args()

    fileregex = re.compile(r"(?<!\w)file=\"(.*?)\"")
    files = {}


    # process all netlists
    for file in args.files:
        location = Path(file.name)
        print(f"parsing {file.name}")

        content = file.read()

        with open(location.name, "w+") as outfile:
            last = 0

            # iterate over all matches
            for match in fileregex.finditer(content):
                start = match.start(1)
                end = match.end(1)
                
                path = Path(match.group(1))
                destpath = path

                # build the destination in the current folder
                # for the retrieved file reference
                if path.is_absolute():
                    try:
                        destpath = path.relative_to(location.parent)
                    except ValueError:
                        destpath = Path(*path.parts[-4:])
                else:
                    path = Path.cwd() / path
                    print(destpath.root)
                    if destpath.parts[0] == "..":
                        destpath = Path(*destpath.parts[1:])
                
                # write the new path the new netlist file
                outfile.write(content[last:start])
                outfile.write(str(destpath))

                # remember file to be copied for later
                files[path] = destpath
                last = end

            outfile.write(content[last:])

    # do dryrun if it is wanted
    if args.d:
        for file in files:
            print(f"copy {file} -> {files[file]}")

        exit(0)


    # copy all files found in netlist earlier
    for file in files:
        src = Path(file)
        dst = Path(files[file])

        if not src.exists():
            print(f"src does not exist: {str(src)}")
            continue

        # create directory
        dst.parent.mkdir(parents=True, exist_ok=True)
        shutil.copyfile(src, dst)
        

if __name__ == "__main__":
    main()